/* -------------------------------------------------------------------
   $Rev: 1292 $ $Date: 2017-07-26 15:12:11 -0400 (Wed, 26 Jul 2017) $
   ------------------------------------------------------------------- */
/* ----------- MERCURY SYSTEMS INC IP PROTECTION HEADER ----------------
* (c) Mercury Systems, Inc. 2020. All rights reserved.
* Mercury Proprietary Information
*
*
* This file, the information contained herein, and its documentation are
* proprietary to Mercury Systems, Inc. This file, the information contained
* herein, and its documentation may only be used, duplicated,  or disclosed
* in accordance with the terms of a written license agreement between the
* receiving party and Mercury or the receiving party and an authorized
* licensee of Mercury.
*
* Each copy of this file shall include all copyrights, trademarks, service
* marks, and proprietary rights notices, if any.
*
* ------------ MERCURY SYSTEMS INC IP PROTECTION HEADER --------------*/

/* -------------------------------------------------------------------
   Description:
      C header file for CAL initialization
   ------------------------------------------------------------------- */

#ifndef CALINI_H
#define CALINI_H

/* -------- */
/* Includes */
/* -------- */
#include "calpolicy.h"
#include "caltypes.h"
#include "calenum.h"

/* ------- */
/* Defines */
/* ------- */

/* ---- ----------- */
/* Type Definitions */
/* ---- ----------- */

/* -------- -------- ---------- */
/* External Function Prototypes */
/* -------- -------- ---------- */
#ifndef CALINI_C
#ifdef __cplusplus
extern "C" {
#endif

/* Published API Functions */
/* --------- --- --------- */
extern SATR CALIni(void);


/* Unpublished Function Prototypes */
/* ----------- -------- ---------- */

/* -------- ------ --------- */
/* External Global Variables */
/* -------- ------ --------- */

#ifdef __cplusplus
}
#endif
#endif
#endif
